@@ -24,14 +24,13 @@ import (
 var Pool *redis.Pool
 
 const (
-	IDLE_TIMEOUT = 60           // ReadinessProbe runs every 30 seconds, this keeps the connection alive between probe intervals.
-	GRAPH_NAME   = "icp-search" // TODO read graph name from config
+	IDLE_TIMEOUT = 60 // ReadinessProbe runs every 30 seconds, this keeps the connection alive between probe intervals.
+	GRAPH_NAME   = "search-db"
 )
 
 // Initializes the pool using functions in this file.
 // Also initializes the Store interface.
 func init() {
-
 	Pool = &redis.Pool{
 		MaxIdle:      10, // TODO: Expose with ENV. Idle connections are connections that have been returned to the pool.
 		MaxActive:    20, // TODO: Expose with ENV. Active connections = connections in-use + idle connections
@@ -39,8 +38,7 @@ func init() {
 		TestOnBorrow: testRedisConnection,
 		Wait:         true,
 	}
-
-	Store = RedisGraphStoreV1{}
+	Store = RedisGraphStoreV2{}
 
 }
 
