@@ -10,14 +10,14 @@ irrespective of what has been deposited with the U.S. Copyright Office.
 package handlers
 
 import (
-	"math/rand"
+	"crypto/rand"
+	"math/big"
 	"time"
 
 	"github.com/golang/glog"
-
 	"github.com/open-cluster-management/search-aggregator/pkg/config"
 	db "github.com/open-cluster-management/search-aggregator/pkg/dbconnector"
-	rg "github.com/open-cluster-management/search-aggregator/pkg/dbconnector"
+	rg2 "github.com/redislabs/redisgraph-go"
 )
 
 var ApplicationLastUpdated time.Time
@@ -27,10 +27,22 @@ func getApplicationUpdateTime() time.Time {
 	return ApplicationLastUpdated
 }
 
+func currAppInstance() int {
+	n, err := rand.Int(rand.Reader, big.NewInt(99999))
+	if err != nil {
+		glog.Error("Not able to generate random number for appInstance")
+		if previousAppInstance == 99999 {
+			return previousAppInstance - 1
+		}
+		return previousAppInstance + 1
+	}
+	return int(n.Int64())
+}
+
 // runs all the specific inter-cluster relationships we want to connect
 func BuildInterClusterEdges() {
 	var tranforms = []struct {
-		transfrom   func() (rg.QueryResult, error)
+		transfrom   func() (rg2.QueryResult, error)
 		description string
 		getUpdate   func() time.Time
 	}{
@@ -56,7 +68,7 @@ func BuildInterClusterEdges() {
 				glog.V(3).Infof("Skipping %s because nothing has changed", edgeFunc.description)
 				continue
 			}
-			glog.V(3).Infof("Running  %s because changed observed", edgeFunc.description)
+			glog.V(3).Infof("Running  %s because change observed", edgeFunc.description)
 			_, err := edgeFunc.transfrom()
 			if err != nil {
 				glog.Errorf("Error %s : %s", edgeFunc.description, err)
@@ -65,7 +77,7 @@ func BuildInterClusterEdges() {
 	}
 }
 
-func getUIDsForSubscriptions() (rg.QueryResult, error) {
+func getUIDsForSubscriptions() (*rg2.QueryResult, error) {
 	query := "MATCH (n {kind: 'subscription'}) RETURN n._uid"
 	uidResults, err := db.Store.Query(query)
 	return uidResults, err
@@ -161,38 +173,66 @@ func getUIDsForSubscriptions() (rg.QueryResult, error) {
 
 }*/
 
-func buildSubscriptions() (rg.QueryResult, error) {
+func buildSubscriptions() (rg2.QueryResult, error) {
 	// Record start time
 	start := time.Now()
-	currentAppInstance := rand.Intn(99999)
+	currentAppInstance := currAppInstance()
 	// Making sure that this instanceID is different from the previous
 	for currentAppInstance == previousAppInstance {
-		currentAppInstance = rand.Intn(99999)
+		currentAppInstance = currAppInstance()
 	}
 
 	// list of remote subscriptions
-	query := "MATCH (n:Subscription) WHERE n.cluster != 'local-cluster' RETURN n._uid, n._hostingSubscription"
+	query := "MATCH (n:Subscription) WHERE n.cluster <> 'local-cluster' RETURN n._uid, n._hostingSubscription"
 	remoteSubscriptions, err := db.Store.Query(query)
 	if err != nil {
-		return rg.QueryResult{}, err
+		return rg2.QueryResult{}, err
 	}
-
-	if len(remoteSubscriptions.Results) > 1 { //Check if any results are returned
+	if !remoteSubscriptions.Empty() { //Check if any results are returned
 		// list of hub subscriptions
 		query = "MATCH (n:Subscription) WHERE  n.cluster='local-cluster' RETURN n._uid, n.namespace+'/'+n.name"
 		hubSubscriptons, err := db.Store.Query(query)
 		if err != nil {
-			return rg.QueryResult{}, err
+			return rg2.QueryResult{}, err
 		}
 
 		//Adding all hubsubscriptions to a map: key is subscription's "namespace+'/'+name", value is UID
 		hubSubMap := make(map[string]string)
-		for _, hubSub := range hubSubscriptons.Results[1:] {
-			hubSubMap[hubSub[1]] = hubSub[0]
+		for hubSubscriptons.Next() {
+			hubRecord := hubSubscriptons.Record()
+			hubSubMap[hubRecord.GetByIndex(1).(string)] = hubRecord.GetByIndex(0).(string)
 		}
 
-		for _, remoteSub := range remoteSubscriptions.Results[1:] {
+		for remoteSubscriptions.Next() {
+			remoteRecord := remoteSubscriptions.Record()
+			var remoteSub [2]string
+
+			if _, ok := remoteRecord.GetByIndex(0).(string); ok {
+				remoteSub[0] = remoteRecord.GetByIndex(0).(string)
+			}
+			if _, ok := remoteRecord.GetByIndex(1).(string); ok {
+				remoteSub[1] = remoteRecord.GetByIndex(1).(string)
+			}
+			var hubSubUID string
+			var ok bool
+			if remoteSub[1] != "" {
+				hubSubUID, ok = hubSubMap[remoteSub[1]]
+			}
+			if ok {
+				// Add an edge between remoteSub and hubSub.
+				query0 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'}), (remoteSub:Subscription {_uid: '%s'}) CREATE (remoteSub)-[:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub)", hubSubUID, remoteSub[0], currentAppInstance)
+				resp, err := db.Store.Query(query0)
+				if err != nil {
+					glog.Errorf("Error %s : %s", query, err) //Logging error so that loop will continue
+				} else {
+					glog.V(4).Info("Number of edges created by query: ", query, " is : ", resp.RelationshipsCreated())
+				}
+			}
 			// remoteSub[1] has the hosting subscription information which is in the format hosting subscription's "namespace+'/'+name"
+			/* NOTE: This logic has been moved to the search-api.  Instead of pre-computing these edges 
+                            here, now we use more complex multi-hop queries to compute these only when needed. This
+                            is more efficient and reduces pressure on RedisGraph.
+                            Keeping this commented code until we validate the new implementation. 
 			if remoteSub[1] != "" {
 				//So, we look up if the hostingSubscription is in the hubSubMap. If it is there, get the UID
 				if hubSubUID, ok := hubSubMap[remoteSub[1]]; ok {
@@ -200,34 +240,44 @@ func buildSubscriptions() (rg.QueryResult, error) {
 					//TODO: For the subscription model, all intercluster edges are named as 'hostedSub {_interCluster: true}'. Change this to relevant names in future
 					//To add edges from hubSub to all resources connected to the remoteSub (bidirectional) - incoming edges and outgoing edges
 					// Add an edge between remoteSub and hubSub. Add edges from hubSub to all resources the remoteSub connects to
-					query1 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'}), (remoteSub {_uid: '%s'})-[]->(n) WHERE n.kind != 'application' AND n.kind != 'subscription' CREATE (remoteSub)-[:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub), (n)-[:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub)", hubSubUID, remoteSub[0], currentAppInstance, currentAppInstance)
+					query1 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'}) MATCH (remoteSub:Subscription {_uid: '%s'})-[]->(n) WHERE n.kind <> 'application' AND n.kind <> 'subscription' CREATE (remoteSub)-[:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub), (n)-[:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub)", hubSubUID, remoteSub[0], currentAppInstance, currentAppInstance)
 					// Add edges from hubSub to all resources that flow into remoteSub eg: pods, deployments, services, replicasets etc.
-					query2 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'}), (remoteSub {_uid: '%s'})<-[]-(n) CREATE (n)-[r:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub)", hubSubUID, remoteSub[0], currentAppInstance )
+					query2 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'}) MATCH (remoteSub:Subscription {_uid: '%s'})<-[]-(n) CREATE (n)-[r:hostedSub {_interCluster: true,app_instance: %d}]->(hubSub)", hubSubUID, remoteSub[0], currentAppInstance)
 					// Connect all resources that flow into remoteSub with the hubsub's channel
-					query3 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'})-[]->(chan) ,  (remoteSub {_uid: '%s'})<-[]-(n)  WHERE chan.kind = 'channel' CREATE (n)-[r:hostedSub {_interCluster: true,app_instance: %d}]->(chan)", hubSubUID, remoteSub[0], currentAppInstance)
+					query3 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'})-[]->(chan:Channel) MATCH  (remoteSub:Subscription {_uid: '%s'})<-[]-(n) CREATE (n)-[r:hostedSub {_interCluster: true,app_instance: %d}]->(chan)", hubSubUID, remoteSub[0], currentAppInstance)
 					// Connect the remoteSub with the hubsub's application
-					query4 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'})<-[]-(app) ,  (remoteSub {_uid: '%s'})  WHERE app.kind = 'application' CREATE (remoteSub)-[:deployedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
+					query4 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'})<-[]-(app:Application) MATCH  (remoteSub:Subscription {_uid: '%s'})  CREATE (remoteSub)-[:deployedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
 					// Connect all resources that flow into remoteSub with the hubsub's application
-					query5 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'})<-[]-(app) ,  (remoteSub {_uid: '%s'})<-[]-(n)  WHERE app.kind = 'application' CREATE (n)-[r:deployedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
+					query5 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'})<-[]-(app:Application) MATCH  (remoteSub:Subscription {_uid: '%s'})<-[]-(n)  CREATE (n)-[r:deployedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
 					// Connect resources that are connected to remoteSub with the hubsub's application - add check to avoid connecting application to itself
-					query6 := db.SanitizeQuery("MATCH (hubSub {_uid: '%s'})<-[]-(app) ,  (remoteSub {_uid: '%s'})-[]->(n)  WHERE app.kind = 'application' AND n.kind != 'application' AND n.kind != 'subscription' CREATE (n)-[r:usedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
+					query6 := db.SanitizeQuery("MATCH (hubSub:Subscription {_uid: '%s'})<-[]-(app:Application) MATCH  (remoteSub:Subscription {_uid: '%s'})-[]->(n)  WHERE n.kind <> 'application' AND n.kind <> 'subscription' CREATE (n)-[r:usedBy {_interCluster: true,app_instance: %d}]->(app)", hubSubUID, remoteSub[0], currentAppInstance)
 
 					queries := [...]string{query1, query2, query3, query4, query5, query6}
 					for _, query := range queries {
-						_, err = db.Store.Query(query)
+						resp, err := db.Store.Query(query)
 						if err != nil {
-							glog.Errorf("Error %s : %s", rg.QueryResult{}, err) //Logging error so that loop will continue
+							glog.Errorf("Error %s : %s", query, err) //Logging error so that loop will continue
+						} else {
+							glog.V(4).Info("Number of edges created by query: ", query, " is : ", resp.RelationshipsCreated())
 						}
 					}
 				}
-			}
+			}*/
 		}
 		//Delete interclusters with other instance ids after all hub subscriptions are processed
-		deleteOldInstance := db.SanitizeQuery("MATCH ()-[e {_interCluster:true}]->() WHERE (type(e)='hostedSub' OR type(e)='usedBy' OR type(e)='deployedBy') AND e.app_instance!=%d DELETE e", currentAppInstance)
+		deleteOldInstance := db.SanitizeQuery("MATCH ()-[e {_interCluster:true}]->() WHERE (type(e)='hostedSub' OR type(e)='usedBy' OR type(e)='deployedBy') AND e.app_instance<>%d DELETE e", currentAppInstance)
 		_, err = db.Store.Query(deleteOldInstance)
 		if err != nil {
-			return rg.QueryResult{}, err
+			return rg2.QueryResult{}, err
 		}
+	} else {
+		//Delete interclusters because there is no remote subscriptions
+		deleteOldInstance := db.SanitizeQuery("MATCH ()-[e {_interCluster:true}]->() WHERE (type(e)='hostedSub' OR type(e)='usedBy' OR type(e)='deployedBy') AND e.app_instance<>%d DELETE e", currentAppInstance)
+		_, err = db.Store.Query(deleteOldInstance)
+		if err != nil {
+			return rg2.QueryResult{}, err
+		}
+
 	}
 	previousAppInstance = currentAppInstance // Next iteration we dont want to use this ID
 	// Record elapsed time
@@ -238,6 +288,5 @@ func buildSubscriptions() (rg.QueryResult, error) {
 	} else {
 		glog.V(4).Infof("Intercluster edge deletion and re-creation took %s", elapsed)
 	}
-
-	return rg.QueryResult{}, nil
+	return rg2.QueryResult{}, nil
 }
